// ParseString.cpp: implementation of the CParseString class
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"
#include "ParseString.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif	// _DEBUG

///////////////////////////////////////////////////////////////////////////////
// CParseString construction

CParseString::CParseString(const CString& str) : CString(str)
{
	if (sizeof(CParseString) == sizeof(str)) {
		m_iErrorCode = ((CParseString)str).GetLastError();
		m_iPos = ((CParseString)str).GetPosition();
	}
	else {
		m_iErrorCode = STR_OK;
		m_iPos = 0;
	}
}

///////////////////////////////////////////////////////////////////////////////
// CParseString operations

void CParseString::IncPosition(int iPos)
{
	m_iPos += iPos;
	if (m_iPos >= GetLength()) m_iPos = GetLength();
}

void CParseString::DecPosition(int iPos)
{
	m_iPos -= iPos;
	if (m_iPos >= GetLength()) m_iPos = GetLength();
}

void CParseString::SetPosition(int iPos)
{
	if (iPos >= GetLength()) m_iPos = GetLength();
	else m_iPos = iPos;
}

CString CParseString::ParseBracket()
{
	int iOpenBrackets = 1;
	int iRightPos;
	int iLen = GetLength();

	m_iErrorCode = STR_OK;
	OverreadLeadingSpaces();
	iRightPos = m_iPos + 1;

	if (m_iPos >= iLen) {
		m_iErrorCode = STR_ERR_EMPTY;
		return "";
	}
	// uPos must index an opening bracket
	if (GetAt(m_iPos) != '(') {
		m_iErrorCode = STR_ERR_LBRACKET;
		return "";
	}

	while (iOpenBrackets > 0) {
		// reached the end of the string => error
		if (iRightPos >= iLen) {
			m_iErrorCode = STR_ERR_RBRACKET;
			return "";
		}

		// look at the next character
		switch (GetAt(iRightPos)) {
			case '(':	iOpenBrackets++;
						break;
			case ')':	iOpenBrackets--;
						break;
		}
		iRightPos++;
	}

	// only the content within the brackets
	CString str = Mid(m_iPos + 1, iRightPos - m_iPos - 2);
	m_iPos = iRightPos;
	return str;
}

CString CParseString::ParseString()
{
	bool bReady = false;
	int iRightPos;
	int iLen = GetLength();

	m_iErrorCode = STR_OK;
	OverreadLeadingSpaces();
	iRightPos = m_iPos + 1;

	// check, if there is a string
	if (m_iPos >= iLen) {
		// the expected string is not there
		m_iErrorCode = STR_ERR_EMPTY;
		return "";
	}

	// search for a ' ' or ')'
	while (!bReady && iRightPos < iLen) {
		switch (GetAt(iRightPos)) {
			case ' ':
			case ')':
			case '\t':
			case '\r':
			case '\n':	bReady = true;
						break;
			default:	iRightPos++;
		}
	}

	// get the string
	CString str = Mid(m_iPos, iRightPos - m_iPos);
	m_iPos = iRightPos + 1;
	return str;
}

char CParseString::ParseChar()
{
	m_iErrorCode = STR_OK;
	OverreadLeadingSpaces();

	if (m_iPos >= GetLength()) {
		m_iErrorCode = STR_ERR_EMPTY;
		return 0;
	}

	return GetAt(m_iPos++);
}

int CParseString::ParseInteger()
{
	bool bSign = false;
	bool bReady = false;
	char cChar;
	int iValue = 0;
	int iLen = GetLength();

	m_iErrorCode = STR_ERR_INT;
	OverreadLeadingSpaces();

	if (m_iPos >= GetLength()) {
		m_iErrorCode = STR_ERR_EMPTY;
		return 0;
	}

	// read in the number
	while (!bReady && m_iPos < iLen) {
		cChar = GetAt(m_iPos);
		switch(cChar) {
			// only one '-' sign expected
			case '-':	if (iValue == 0) bSign = !bSign;
			case '+':	if (iValue == 0) m_iPos++;
						else bReady = true;
						break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':	iValue = 10 * iValue + cChar - '0';
						m_iErrorCode = STR_OK;
						m_iPos++;
						break;
			case ')':
			case ' ':	m_iPos++;
			// no break here
			default:	bReady = true;
		}
	}

	// convert the number if neccessary
	if (bSign) iValue = -iValue;
	return iValue;
}

double CParseString::ParseDouble()
{
	bool bSign = false;
	bool bReady = false;
	bool bDot = false;
	char cChar;
	double dValue = 0.0;
	double dNumber;
	double dDivisor = 1.0;
	int iLen = GetLength();
	int iExp = 0;

	m_iErrorCode = STR_ERR_DOUBLE;
	OverreadLeadingSpaces();

	if (m_iPos >= GetLength()) {
		m_iErrorCode = STR_ERR_EMPTY;
		return 0.0;
	}

	// read in the number
	while (!bReady && m_iPos < iLen) {
		cChar = GetAt(m_iPos);
		switch(cChar) {
			case '-':	if (dValue == 0) bSign = !bSign;
			case '+':	if (dValue == 0) m_iPos++;
						else bReady = true;
						break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':	dValue = 10.0 * dValue + (double)(cChar - '0');
						dDivisor *= 10.0;
						m_iErrorCode = STR_OK;
						m_iPos++;
						break;
			case '.':
			case ',':	dDivisor = 1.0;
						bDot = true;
						m_iPos++;
						break;
			case 'e':
			case 'E':	m_iPos++;
						iExp = ParseInteger();	// exponent
						if (m_iErrorCode != STR_OK) {
							dValue = 0.0;
							dDivisor = 1.0;
							m_iErrorCode = STR_ERR_DOUBLE;
						}
						else dDivisor *= pow(10,-iExp);
						bReady = true;
						break;
			case ')':
			case ' ':	m_iPos++;
			// no break here
			default:	bReady = true;
		}
	}

	// convert the number if neccessary
	if (bDot) dNumber = dValue / dDivisor;
	else dNumber = dValue;

	if (bSign) dNumber = -dNumber;
	return dNumber;
}

const CParseString& CParseString::operator=(const CString& str)
{
	CString::operator=(str);
	if (sizeof(CParseString) == sizeof(str)) {
		m_iErrorCode = ((CParseString)str).GetLastError();
		m_iPos = ((CParseString)str).GetPosition();
	}
	else {
		m_iErrorCode = STR_OK;
		m_iPos = 0;
	}
	return *this;
}

void CParseString::OverreadLeadingSpaces()
{
	int iLen = GetLength();

	// overread inital spaces, tabs, returns and new lines
	while (m_iPos < iLen) {
		switch (GetAt(m_iPos)) {
			case ' ':
			case '\t':
			case '\r':
			case '\n':	m_iPos++;
						break;
			default:	return;
		}
	}
}
