// Perception.h: interface for the CPerception class
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_PERCEPTION_H_
#define __AFX_PERCEPTION_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

// Perception.h header file declares following classes

class CPerception;
class CPlayerAgent;

#include "VisibleObject.h"

#define PLAYER_COUNT	22		// maximum player count in the match
#define FLAG_COUNT		55		// maximum flags on the field (goals included)

#define MAX_TEAMMATE_DIST	20.0

// no of stored future positions of the ball
#define MAX_FUTURE_POSITIONS	30
#define EVAL_MAX_ITERATIONS	30
#define EVAL_POWER_STEP	4

///////////////////////////////////////////////////////////////////////////////
// Class: CPerception
// Date: 14.04.01
// Version Date: 11.05.01
// Version: 1.29
// Author: Marek Lehuta
//
// Base Class: CObject
///////////////////////////////////////////////////////////////////////////////
// Description: represents agent's perceptions of visible objects
//				contains the methods for manipulating the perceptions
///////////////////////////////////////////////////////////////////////////////

class CPerception : public CObject
{
// Construction
public:
	CPerception();

// Attributes
private:
	int m_iSimTime;				// simulation time of the server
	int m_iSeeTime;				// time received from the last perception
	int m_iHearTime;			// time received from the last hear message
	int m_iSenseTime;			// time received from the last sense_body
	int m_iPlayMode;			// mode the game is in (before_kickoff, ...)

	int m_iPlayerCount;			// number of currently visible players
	int m_iFlagCount;			// number of currently visible flags

	CPlayer* m_pPlayer[PLAYER_COUNT];	// max 22 players ([0] is agent's player)
	CPlayer* m_pUnknownPlayer[PLAYER_COUNT - 1];	// players of unknown side
	CBall* m_pBall;				// the ball
	CGoal* m_pGoal[2];			// [0] is the left goal
	CFlag* m_pFlag[FLAG_COUNT];	// flags of the corners and midline
	CLine* m_pLine[4];

	CPlayer* m_pTmpPlayer[PLAYER_COUNT - 1];	// temporary players just for parsing

	// player info
	CString m_strTeam;			// name of the team the player is in

	// sense body information
	int m_iViewQuality;
	int m_iViewAngle;
	int m_iStamina;
	double m_dEffort;
	double m_dSpeed;
	double m_dHeadAng;
	int m_iKickCount;
	int m_iDashCount;
	int m_iTurnCount;
	int m_iSayCount;
	int m_iTurnNeckCount;

	bool m_bNewPerception;		//kvoli hladaniu lopty, aby sme vedeli, ci bol vnem

	bool m_bDebug;				// true, if debug output is switched on
	FILE* m_pFile;				// debug file structure

// Operations
public:
	int GetSimTime() const { return m_iSimTime; };
	int GetSeeTime() const { return m_iSeeTime; };
	int GetHearTime() const { return m_iHearTime; };
	int GetSenseTime() const { return m_iSenseTime; };
	int GetPlayMode() const { return m_iPlayMode; };
	bool WasPerception() const { return m_iSimTime == m_iSeeTime; };
//	int GetPlayerCount() const { return m_iPlayerCount; };
//	int GetFlagCount() const { return m_iFlagCount; };

	CPlayer* GetAgentPlayer() const { return m_pPlayer[0]; };
	CPlayer** GetPlayers() const { return (CPlayer**)m_pPlayer; };
	CPlayer** GetTeammates() const { return (CPlayer**)&m_pPlayer[1]; };
	CPlayer** GetOpponents() const { return (CPlayer**)&m_pPlayer[11]; };
	CPlayer** GetUnknownPlayers() const;
	CBall* GetBall() const { return m_pBall; };
	CGoal* GetGoal(bool bOur = false);
	CFlag** GetFlags() const { return (CFlag**)m_pFlag; };
	CLine** GetLines() const { return (CLine**)m_pLine; };

	void SetTeam(const CString& strTeam) { m_strTeam = strTeam; };
	CString GetTeam() const { return m_strTeam; };

	int GetSide() const { return m_pPlayer[0]->GetSide(); };
	int GetNumber() const { return m_pPlayer[0]->GetNumber(); };
//	void SetSide(int side) { m_iSide = side; m_pThisPlayer->SetTeam(TRUE, side); };
//	void SetNumber(int number) { m_iNumber = number; };

	int GetViewQuality() const { return m_iViewQuality; };
	int GetViewAngle() const { return m_iViewAngle; };
	double GetStamina() const { return m_iStamina; };
	double GetEffort() const { return m_dEffort; };
	int GetKickCount() const { return m_iKickCount; };
	int GetDashCount() const { return m_iDashCount; };
	int GetTurnCount() const { return m_iTurnCount; };
	int GetSayCount() const { return m_iSayCount; };
	int GetTurnNeckCount() const { return m_iTurnNeckCount; };

	void SetNewPerception(bool bNewPerception) { m_bNewPerception = bNewPerception; };
	bool GetNewPerception() const { return m_bNewPerception; };

	void SetDebug(bool bDebug, FILE* pFile) { m_bDebug = bDebug; m_pFile = pFile; };

	// proposition functions
	double GetBallDistance(CPlayer* pPlayer = NULL) const;
	double GetBallDirection(CPlayer* pPlayer = NULL) const;
	double GetGoalDistance(CPlayer* pPlayer = NULL, bool bOwn = false);
	double GetGoalDirection(CPlayer* pPlayer = NULL, bool bOwn = false);

	bool IsInPenaltyArea(CMovableObject* pObject = NULL, bool bOur = true) const;
	bool IsOnOurHalf(CMovableObject* pObject = NULL) const;
//	bool IsOnOwnHalf(CPlayer* pPlayer = NULL) const;

//	CPosition* GetBallFuturePos(int iTime) { return m_pBall->GetFuturePos(iTime); };
	CPlayer* GetBallNearestPlayer() const;
	CPlayer* GetBallNearestTeammate() const;
	CPlayer* GetBallNearestOpponent() const;
	int GetBallInterceptTime(CPlayer* pPlayer = NULL) const;
	CPosition GetBallInterceptPos(CPlayer* pPlayer = NULL) const;

	bool IsOurPlayer(CPlayer* pPlayer) const;
	CPlayer* GetNearestTeammateTo(const CPosition& pos) const;
	CPlayer* GetNearestOpponentTo(const CPosition& pos) const;
	CPlayer* GetNearestTeammate(CPlayer* pPlayer = NULL) const;
	CPlayer* GetNearestOpponent(CPlayer* pPlayer = NULL) const;
	CPlayer* GetBetterPlacedTeammate();
	double GetAvoidAngle(const CPosition& destPos, CPlayer* pPlayer = NULL) const;

	bool ParseMessage(LPCTSTR psMsg);
	void UpdateAgent(LPCTSTR psMsg);
	void UpdateField();
	void UnlockAll();

	void SetPlayerAgent(CPlayerAgent *pPA) { m_pPlayerAgent = pPA; };
	double EvaluatePower(double distance);
	double EvaluateBallDistance(double power);
  double GetFreeDirection(double lAngle, double rAngle);
  CPlayer* GetNearestOpponent(CPlayer* pPlayer, double direction, double tolerance);

  CPlayerAgent *m_pPlayerAgent;

private:
	bool ParseInit(CParseString& msg);
	bool ParseSee(CParseString& msg);
	bool ParseHear(CParseString& msg);
	bool ParseSenseBody(CParseString& msg);
	bool ParseObject(CParseString& msg);
	void InvalidateAll();
	bool LocalizeAgent();
	bool LocalizeAll();
	void StorePlayer(CPlayer& player);

// Implementation
public:
	virtual ~CPerception();

};

///////////////////////////////////////////////////////////////////////////////

#endif	// __AFX_PERCEPTION_H_
