// VisibleObject.cpp: implementation of the CVisibleObject class
//						and its descendants
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"
#include "VisibleObject.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif	// _DEBUG

extern double gFlagXPos[FLAG_COUNT];
extern double gFlagYPos[FLAG_COUNT];
extern char* gPlayMode[PLAYMODE_COUNT];

///////////////////////////////////////////////////////////////////////////////
// CVisibleObject class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CVisibleObject construction

CVisibleObject::CVisibleObject(double dAbsX, double dAbsY)
	: m_absPos(dAbsX, dAbsY)
{
	m_bValid = true;
}

CVisibleObject::CVisibleObject(const CVisibleObject& obj)
	: m_absPos(obj.GetAbsPos()), m_relPos(obj.GetRelPos())
{
	m_bValid = obj.IsValid();
}

///////////////////////////////////////////////////////////////////////////////
// CVisibleObject operations

void CVisibleObject::SetAbsPos(double dAbsX, double dAbsY)
{
	m_absPos.SetXY(dAbsX, dAbsY);
	m_bValid = true;
}

void CVisibleObject::SetRelVector(double dDist, double dDir)
{
	m_relPos.SetVector(dDist, dDir);
	m_bValid = true;
}

void CVisibleObject::SetRelPos(double dRelX, double dRelY)
{
	m_relPos.SetXY(dRelX, dRelY);
	m_bValid = true;
}

///////////////////////////////////////////////////////////////////////////////
// CVisibleObject overrides

const CVisibleObject& CVisibleObject::operator=(const CVisibleObject& obj)
{
	m_bValid = obj.IsValid();
	m_absPos = obj.GetAbsPos();
	m_relPos = obj.GetRelPos();
	return *this;
}

void CVisibleObject::ParseInfo(CParseString& str)
{
	double dDist, dDir;

	dDist = str.ParseDouble();
	dDir = str.ParseDouble();
	m_relPos.SetVector(dDist, dDir);
	m_bValid = true;
}

///////////////////////////////////////////////////////////////////////////////
// CMovableObject class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CMovableObject construction

CMovableObject::CMovableObject(double dAbsX, double dAbsY)
	: CVisibleObject(dAbsX, dAbsY)
{
	Init();
}

CMovableObject::CMovableObject(const CPosition& aPos, double dDistChng, double dDirChng)
	: CVisibleObject(aPos)
{
	m_bLock = true;
	m_iSeeTime = m_iSimTime = 0;
	m_bChngValid = true;
	m_dDistChng = dDistChng;
	m_dDirChng = dDirChng;
}

CMovableObject::CMovableObject(const CVector& rPos, double dDistChng, double dDirChng)
	: CVisibleObject(rPos)
{
	m_bLock = true;
	m_iSeeTime = m_iSimTime = 0;
	m_bChngValid = true;
	m_dDistChng = dDistChng;
	m_dDirChng = dDirChng;
}

CMovableObject::CMovableObject(const CPosition& aPos, const CVector& speed)
	: CVisibleObject(aPos), m_speed(speed)
{
	Init();
}

CMovableObject::CMovableObject(const CMovableObject& obj)
	: CVisibleObject(obj), m_oldPos(obj.GetOldPos()),
		m_speed(obj.GetSpeed())
{
	m_bLock = obj.IsLocked();
	m_iSeeTime = obj.GetSeeTime();
	m_iSimTime = obj.GetSimTime();
	m_bChngValid = obj.IsChngValid();
	m_dDistChng = obj.GetDistChng();
	m_dDirChng = obj.GetDirChng();
}

void CMovableObject::Init()
{
	m_bLock = true;
	m_iSeeTime = m_iSimTime = 0;
	m_bChngValid = false;
	m_dDistChng = m_dDirChng = 0.0;
}

///////////////////////////////////////////////////////////////////////////////
// CMovableObject operations

void CMovableObject::SetTime(int iTime)
{
	m_iSeeTime = m_iSimTime = iTime;
	m_bValid = true;
}

void CMovableObject::SetAbsPos(double dAbsX, double dAbsY)
{
	m_oldPos = m_absPos;
	CVisibleObject::SetAbsPos(dAbsX, dAbsY);
	m_speed = m_absPos - m_oldPos;
	if (m_speed.GetLength() > GetSpeedMax()) m_speed.SetLength(GetSpeedMax());
	m_bChngValid = false;
}

void CMovableObject::SetAbsPos(const CPosition& pos)
{
	m_oldPos = m_absPos;
	CVisibleObject::SetAbsPos(pos);
	m_speed = m_absPos - m_oldPos;
	if (m_speed.GetLength() > GetSpeedMax()) m_speed.SetLength(GetSpeedMax());
	m_bChngValid = false;
}

void CMovableObject::SetSpeedVector(double dDist, double dDir)
{
	m_speed.SetVector(dDist, dDir);
	m_bValid = true;
	m_bChngValid = false;
}

void CMovableObject::SetSpeed(double dSpX, double dSpY)
{
	m_speed.SetXY(dSpX, dSpY);
	m_bValid = true;
	m_bChngValid = false;
}

void CMovableObject::SetSpeed(const CVector& speed)
{
	m_speed = speed;
	m_bValid = true;
	m_bChngValid = false;
}

double CMovableObject::GetDistanceTo(const CVisibleObject* pObj) const
{
	return GetDistanceTo(pObj->GetAbsPos());
}

double CMovableObject::GetDistanceTo(const CPosition& pos) const
{
	return (m_absPos - pos).GetLength();
}

CPosition CMovableObject::GetFuturePos(int iTime) const
{
	CVector speed(m_speed);
	CPosition pos(m_absPos);

	while (iTime-- > 0) {
		pos += speed;
		speed.Magnify(GetDecay());
	}
	return pos;
}

void CMovableObject::UpdatePos(const CPosition& refPos)
{
	m_iSimTime++;
	if (m_iSimTime - m_iSeeTime > GetMaxAge()) {
		Invalidate();
		return;
	}

	m_oldPos = m_absPos;
	m_absPos += m_speed;
	m_relPos = m_absPos - refPos;
	m_speed.Magnify(GetDecay());
	m_bChngValid = false;
}

void CMovableObject::UpdateState(const CMovableObject& obj)
{
	if (m_bChngValid) return;
	m_oldPos = obj.GetAbsPos();
	m_speed = (m_absPos - m_oldPos) / obj.GetAge();
	if (m_speed.GetLength() > GetSpeedMax()) m_speed.SetLength(GetSpeedMax());
}

void CMovableObject::CalcAbsPos(const CPosition& refPos)
{
	m_oldPos = m_absPos;
	m_absPos = refPos + m_relPos;
}

void CMovableObject::CalcSpeed(const CPosition& refPos)
{
	CPosition futAbsPos(refPos);
	CVector futRelPos(m_relPos);

	futRelPos.Rotate(m_dDirChng);
	futRelPos.Stretch(m_dDistChng);
	futAbsPos += futRelPos;
	m_speed = futAbsPos - m_absPos;
}

///////////////////////////////////////////////////////////////////////////////
// CMovableObject overrides

const CMovableObject& CMovableObject::operator=(const CMovableObject& obj)
{
	CVisibleObject::operator=(obj);
	m_bLock = obj.IsLocked();
	m_iSeeTime = obj.GetSeeTime();
	m_iSimTime = obj.GetSimTime();
	m_bChngValid = obj.IsChngValid();
	m_dDistChng = obj.GetDistChng();
	m_dDirChng = obj.GetDirChng();
	m_oldPos = obj.GetOldPos();
	m_speed = obj.GetSpeed();
	return *this;
}

void CMovableObject::ParseInfo(CParseString& str)
{
	CVisibleObject::ParseInfo(str);
	m_bLock = true;
	m_dDistChng = str.ParseDouble();
	m_dDirChng = str.ParseDouble();
	if (str.GetLastError() == STR_OK) m_bChngValid = true;
	else m_bChngValid = false;
}

/*
CVector CMovableObject::GetRelSpeed(const CPosition& pos, double dDir)
{
	// calculate according to the absolute direction the agent is facing
	TVector posObject = m_tPosition;
	TVector posSpeed = posObject + m_tAbsSpeed;

	// translate to agents coordinate system
	posObject -= pos;
	posObject.Rotate(dir);
	posSpeed -= pos;
	posSpeed.Rotate(dir);
	return posSpeed - posObject;
}
*/
///////////////////////////////////////////////////////////////////////////////
// CPlayer Class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CPlayer construction

CPlayer::CPlayer()
{
	m_iSide = SIDE_UNKNOWN;
	m_iNumber = NUMBER_UNKNOWN;
	m_dBodyDir = m_dHeadDir = 0.0;
}

CPlayer::CPlayer(const CPlayer& player) : CMovableObject(player)
{
	m_iSide = player.GetSide();
	m_iNumber = player.GetNumber();
	m_bDirValid = player.IsDirValid();
	m_dBodyDir = player.GetBodyDir();
	m_dHeadDir = player.GetHeadDir();
}

///////////////////////////////////////////////////////////////////////////////
// CPlayer operations

void CPlayer::SetBodyDir(double dBodyDir)
{
	CVector vect(1.0, dBodyDir);

	m_dBodyDir = vect.GetAngle();
}

void CPlayer::SetHeadDir(double dHeadDir)
{
	if (dHeadDir > 90.0) m_dHeadDir = 90.0;
	else if (dHeadDir < -90.0) m_dHeadDir = -90.0;
	else m_dHeadDir = dHeadDir;
}

double CPlayer::GetAngleTo(const CVisibleObject* pObj) const
{
	return GetAngleTo(pObj->GetAbsPos());
}

double CPlayer::GetAngleTo(const CPosition& pos) const
{
	CVector dir = pos - m_absPos;

	dir.Rotate(-m_dBodyDir);

	return dir.GetAngle();
}

double CPlayer::GetHeadAngleTo(const CVisibleObject* pObj) const
{
	return GetHeadAngleTo(pObj->GetAbsPos());
}

double CPlayer::GetHeadAngleTo(const CPosition& pos) const
{
	CVector dir = pos - m_absPos;

	dir.Rotate(-m_dBodyDir);

	return dir.GetAngle() - m_dHeadDir;
}
/*
CPosition CPlayer::GetNextFuturePosition() const
{
	m_tFutureSpeed += m_tFutureAcc;
	m_tFuturePos += m_tFutureSpeed;
	m_tFutureSpeed *= GetDecay();
}
*/
double CPlayer::GetNextFutureDistance() const
{
/*
	m_tFutureSpeed += m_tFutureAcc;
	m_tFuturePos += m_tFutureSpeed;
	m_tFutureSpeed *= GetDecay();
	return m_tFuturePos.GetLength();
*/
	return 0;
}
/*
CPosition CPlayer::GetFuturePositionActive(int iTime, double dPower, double dDir, double dEffort) const
{
	int		i;
	double	dAcc = DASH_POWER_RATE * effort * dashpower;

	m_tFutureSpeed = m_tAbsSpeed;
	m_tFutureAcc.SetVectorByDirection(dAcc, direction);
	m_tFuturePos = m_tPosition;
	
	for (i = 0; i < timeAhead; i++)
		GetNextFuturePosition();

	pos = m_tFuturePos;
}
*/
double CPlayer::GetFutureDistanceActive(int iTime, double dPower, double dDir, double dEffort) const
{
//	CVector dist, speed(dPower / 100.0, 0.0);
	CVector dist, speed(dPower * DASH_POWER_RATE, 0.0);

	while (iTime-- > 0) dist += speed;

	return dist.GetLength();

}

double CPlayer::GetFutureDistanceActive(int iTime, double dPower, bool bIsTurn) const
{
//	CVector dist, speed(dPower / 100.0, 0.0);
	CVector dist, speed(dPower * DASH_POWER_RATE, 0.0);

	while (--iTime > 0) dist += speed;

	return dist.GetLength();
/*
	int		i;
	double	dAcc = DASH_POWER_RATE * effort * dashpower;
	double	ret = 0.0;

	m_tFutureSpeed = m_tAbsSpeed;
	m_tFutureAcc.SetVectorByDirection(dAcc, dir);
	m_tFuturePos = TVector(0.0, 0.0);
	
	for (i = 0; i < timeAhead; i++)
		ret = GetNextFutureDistance();
	return ret;
*/
}

void CPlayer::UpdateState(const CPlayer& player)
{
	CMovableObject::UpdateState(player);
	if (m_iSide == SIDE_UNKNOWN) m_iSide = player.GetSide();
	if (m_iNumber == NUMBER_UNKNOWN) m_iNumber = player.GetNumber();
	if (m_bDirValid) return;
	m_dBodyDir = m_speed.GetAngle();
	m_dHeadDir = 0.0;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPlayer::CheckPreventOffside(TPlayer* other)
Date    : 31.07.98
-------------------------------------------------------------------------------
Parameter: pointer to a player of the opponent team
Returnval: TRUE, if the player passed is nearer to his own goal, or if this player
			  is in its own half which prevents this player to be in offside
Description: checks, if the other player prevents me from beeing offside
-----------------------------------------------------------------------------
BOOL TPlayer::CheckPreventOffside(TPlayer* other)
{
	if (IsInOwnHalfOfField())
		return TRUE;

	if (m_iTeam == LEFT_TO_RIGHT)
	{
		if (GetAbsX() <= other->GetAbsX())
			return TRUE;
	}
	else
	{
		if (GetAbsX() >= other->GetAbsX())
			return TRUE;
	}
	return FALSE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPlayer::GetInLeftSideOfField()
Date    : 27.05.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: TRUE, if the player is on the left side of the field (seen in playdirection)
Description: _
-----------------------------------------------------------------------------
BOOL TPlayer::IsOnLeftSideOfField()
{
	return ((m_iTeam == LEFT_TO_RIGHT) && (GetAbsY() < 0.0))
				|| ((m_iTeam == RIGHT_TO_LEFT) && (GetAbsY() > 0.0));
}

BOOL TPlayer::IsInOwnHalfOfField()
{
	return ((m_iTeam == LEFT_TO_RIGHT) && (GetAbsX() < 0.0))
				|| ((m_iTeam == RIGHT_TO_LEFT) && (GetAbsX() > 0.0));
}

/*-----------------------------------------------------------------------------
Function: void	InitFreePassArea(TDirection iSecureBoundary)
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: secure boundary in which the ball flies safe
Returnval: (void)
Description: initialize the two bordes at max values
-----------------------------------------------------------------------------
void TPlayer::InitFreePassArea(int iSecureBoundary)
{
	m_iFreeAreaBordersLeft=m_iDirection-iSecureBoundary;
	m_iFreeAreaBordersRight=m_iDirection+iSecureBoundary;
}

/*-----------------------------------------------------------------------------
Function: void	UpdateFreePassArea(TPlayer *pOpponent)
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: Pointer TPlayer who covers the player
			secure boundary angle by which it is save
Returnval: (void)
Description: calculates the free pass area by the opponent who blocks
				its ensured that the player is blocked by that opponent
-----------------------------------------------------------------------------
//void TPlayer::UpdateFreePassArea(TPlayer* pOpponent, int iSecureBoundary)
void TPlayer::UpdateFreePassArea(TDirection pDir, int iSecureBoundary)
{
	if(m_iFreeAreaBordersLeft < m_iFreeAreaBordersRight)
	{
		if (pDir < this->GetDirection())
			m_iFreeAreaBordersLeft = pDir + iSecureBoundary;
		else
			m_iFreeAreaBordersRight = pDir - iSecureBoundary;

		if (m_iFreeAreaBordersLeft > m_iFreeAreaBordersRight)
			m_iFreeAreaBordersLeft = m_iFreeAreaBordersRight;
	}
}

/*-----------------------------------------------------------------------------
Function: int	GetFreePassArea()
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: none
Returnval: int
Description: gives the range of the free pass area
-----------------------------------------------------------------------------
int	TPlayer::GetFreePassArea()
{
	return  m_iFreeAreaBordersRight - m_iFreeAreaBordersLeft;
}

/*-----------------------------------------------------------------------------
Function: int	GetFreePassDirection()
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: none
Returnval: int
Description: gives the angle of the free pass direction
-----------------------------------------------------------------------------
int	TPlayer::GetFreePassDirection()
{
	return m_iFreeAreaBordersLeft + (GetFreePassArea() / 2);
}
*/
///////////////////////////////////////////////////////////////////////////////
// CPlayer overrides

const CPlayer& CPlayer::operator=(const CPlayer& player)
{
	CMovableObject::operator=(player);
	m_iSide = player.GetSide();
	m_iNumber = player.GetNumber();
	m_bDirValid = player.IsDirValid();
	m_dBodyDir = player.GetBodyDir();
	m_dHeadDir = player.GetHeadDir();
	return *this;
}

void CPlayer::ParseInfo(CParseString& str)
{
	CParseString strID = str.ParseBracket();

	strID.ParseString();
	strID.ParseString();
	m_iSide = SIDE_UNKNOWN;
	m_iNumber = strID.ParseInteger();
	CMovableObject::ParseInfo(str);
	m_dBodyDir = str.ParseDouble();
	m_dHeadDir = str.ParseDouble();
	if (str.GetLastError() == STR_OK) m_bDirValid = true;
	else m_bDirValid = false;
}

///////////////////////////////////////////////////////////////////////////////
// CBall Class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CBall overrides

const CBall& CBall::operator=(const CBall& ball)
{
	CMovableObject::operator=(ball);
	return *this;
}

void CBall::ParseInfo(CParseString& str)
{
	str.ParseBracket();
	CMovableObject::ParseInfo(str);
}

///////////////////////////////////////////////////////////////////////////////
// CGoal Class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CGoal operations

void CGoal::CalcAbsPos()
{
	m_bValid = true;
	switch (m_iID) {
		case GOAL_LEFT:	m_absPos.SetXY(gFlagXPos[L_GOAL], gFlagYPos[L_GOAL]);
						break;
		case GOAL_RIGHT:	m_absPos.SetXY(gFlagXPos[R_GOAL], gFlagYPos[R_GOAL]);
							break;
		default: m_bValid = false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// CGoal overrides

const CGoal& CGoal::operator=(const CGoal& goal)
{
	CVisibleObject::operator=(goal);
	m_iID = goal.GetID();
	return *this;
}

void CGoal::ParseInfo(CParseString& str)
{
	CParseString strID = str.ParseBracket();

	strID.ParseString();
	switch (strID.ParseChar()) {
		case 'l':	m_iID = GOAL_LEFT;
					break;
		case 'r':	m_iID = GOAL_RIGHT;
					break;
		default:	m_iID = GOAL_UNKNOWN;
	}
	CVisibleObject::ParseInfo(str);
	CalcAbsPos();
}

///////////////////////////////////////////////////////////////////////////////
// CFlag Class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CFlag construction

CFlag::CFlag(int iAreaID, int iPriID, int iSecID, int iTerID)
{
	m_iAreaID = iAreaID;
	m_iPriID = iPriID;
	m_iSecID = iSecID;
	m_iTerID = iTerID;
	CalcAbsPos();
}

CFlag::CFlag(const CFlag& flag) : CVisibleObject(flag)
{
	m_iAreaID = flag.GetAreaID();
	m_iPriID = flag.GetPriID();
	m_iSecID = flag.GetSecID();
	m_iTerID = flag.GetTerID();
}

///////////////////////////////////////////////////////////////////////////////
// CFlag operations

void CFlag::SetID(int iAreaID, int iPriID, int iSecID, int iTerID)
{
	m_iAreaID = iAreaID;
	m_iPriID = iPriID;
	m_iSecID = iSecID;
	m_iTerID = iTerID;
	CalcAbsPos();
}

void CFlag::CalcAbsPos()
{
	int index;

	m_bValid = true;
	index = (m_iAreaID - 1) * 6;
	if (m_iAreaID == FLAG_FIELD) {	// flag is somewhere in the field
		index += (m_iPriID - 4) * 9;
		if (m_iPriID > 5) index += (m_iPriID - 6) * 2;
		switch (m_iPriID) {
			case FLAG_LEFT:
			case FLAG_RIGHT:	index += (m_iSecID - 6) * 4;
								index += m_iTerID / 10;
								break;
			case FLAG_TOP:
			case FLAG_BOTTOM:	index += (m_iSecID - 4) * 5;
								index += (m_iTerID - 10) / 10;
								break;
			case FLAG_CENTER:	index += m_iSecID - 6;
								break;
			default: m_bValid = false;
		}
	}
	else index += (m_iPriID - 4) * 3 + m_iSecID - 6;	// flag is either goal or penalty area
	m_absPos.SetXY(gFlagXPos[index], gFlagYPos[index]);
}

///////////////////////////////////////////////////////////////////////////////
// CFlag overrides

const CFlag& CFlag::operator=(const CFlag& flag)
{
	CVisibleObject::operator=(flag);
	m_iAreaID = flag.GetAreaID();
	m_iPriID = flag.GetPriID();
	m_iSecID = flag.GetSecID();
	m_iTerID = flag.GetTerID();
	return *this;
}

void CFlag::ParseInfo(CParseString& str)
{
	char cChar;
	CParseString strID = str.ParseBracket();

	strID.ParseString();
	switch (cChar = strID.ParseChar()) {
		case 'g':	m_iAreaID = FLAG_GOAL;
					cChar = strID.ParseChar();
					break;
		case 'p':	m_iAreaID = FLAG_PENALTY;
					cChar = strID.ParseChar();
					break;
		default:	m_iAreaID = FLAG_FIELD;
	}
	if (cChar == 'c') {
		m_iPriID = FLAG_CENTER;
		switch (strID.ParseChar()) {
			case 't':	m_iSecID = FLAG_TOP;
						break;
			case 'b':	m_iSecID = FLAG_BOTTOM;
						break;
			default:	m_iSecID = FLAG_CENTER;
		}
	}
	else {
		switch (cChar) {
			case 'l':	m_iPriID = FLAG_LEFT;
						m_iSecID = FLAG_CENTER;	// just for correct positioning
						break;
			case 'r':	m_iPriID = FLAG_RIGHT;
						m_iSecID = FLAG_CENTER;	// just for correct positioning
						break;
			case 't':	m_iPriID = FLAG_TOP;
						m_iSecID = FLAG_TOP;	// just for correct positioning
						break;
			case 'b':	m_iPriID = FLAG_BOTTOM;
						m_iSecID = FLAG_TOP;	// just for correct positioning
						break;
			default:	m_iPriID = FLAG_UNKNOWN;
		}
		m_iTerID = strID.ParseInteger();
		if (strID.GetLastError() != STR_OK) {
			switch (strID.ParseChar()) {
				case 'l':	m_iSecID = FLAG_LEFT;
							break;
				case 'r':	m_iSecID = FLAG_RIGHT;
							break;
				case 't':	m_iSecID = FLAG_TOP;
							break;
				case 'b':	m_iSecID = FLAG_BOTTOM;
							break;
				case 'c':	m_iSecID = FLAG_CENTER;
							break;
				default:	m_iSecID = FLAG_UNKNOWN;
			}
			m_iTerID = strID.ParseInteger();
		}
		else if (m_iSecID == FLAG_TOP) m_iTerID += 10;	// this is for correct positioning
	}
	CVisibleObject::ParseInfo(str);
	CalcAbsPos();
}

///////////////////////////////////////////////////////////////////////////////
// CLine Class
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// CLine construction

CLine::CLine(const CLine& line) : CVisibleObject(line)
{
	m_iID = line.GetID();
	m_endPos = line.GetEndPos();
}

///////////////////////////////////////////////////////////////////////////////
// CLine operations

void CLine::SetEndPos(double dEndX, double dEndY)
{
	m_endPos.SetXY(dEndX, dEndY);
	m_bValid = true;
}

void CLine::CalcAbsPos()
{
	m_bValid = true;
	switch (m_iID) {
		case LINE_LEFT:	m_absPos.SetXY(gFlagXPos[LB_FLAG], gFlagYPos[LB_FLAG]);
						m_endPos.SetXY(gFlagXPos[LT_FLAG], gFlagYPos[LT_FLAG]);
						break;
		case LINE_RIGHT:	m_absPos.SetXY(gFlagXPos[RT_FLAG], gFlagYPos[RT_FLAG]);
							m_endPos.SetXY(gFlagXPos[RB_FLAG], gFlagYPos[RB_FLAG]);
							break;
		case LINE_TOP:	m_absPos.SetXY(gFlagXPos[LT_FLAG], gFlagYPos[LT_FLAG]);
						m_endPos.SetXY(gFlagXPos[RT_FLAG], gFlagYPos[RT_FLAG]);
						break;
		case LINE_BOTTOM:	m_absPos.SetXY(gFlagXPos[RB_FLAG], gFlagYPos[RB_FLAG]);
							m_endPos.SetXY(gFlagXPos[LB_FLAG], gFlagYPos[LB_FLAG]);
							break;
		default: m_bValid = false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// CLine overrides

const CLine& CLine::operator=(const CLine& line)
{
	CVisibleObject::operator=(line);
	m_iID = line.GetID();
	m_endPos = line.GetEndPos();
	return *this;
}

void CLine::ParseInfo(CParseString& str)
{
	CParseString strID = str.ParseBracket();

	strID.ParseString();
	switch (strID.ParseChar()) {
		case 'l':	m_iID = LINE_LEFT;
					break;
		case 'r':	m_iID = LINE_RIGHT;
					break;
		case 't':	m_iID = LINE_TOP;
					break;
		case 'b':	m_iID = LINE_BOTTOM;
					break;
		default:	m_iID = LINE_UNKNOWN;
	}
	CVisibleObject::ParseInfo(str);
	CalcAbsPos();
}
