// VisibleObject.h: interface for the CVisibleObject class and its descendants
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_VISIBLEOBJECT_H_
#define __AFX_VISIBLEOBJECT_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

// VisibleObject.h header file declares following classes

class CVisibleObject;
class CMovableObject;
class CPlayer;
class CBall;
class CGoal;
class CFlag;
class CLine;

#include "Vector.h"
#include "Position.h"
#include "ParseString.h"

// constants for CPlayer class

#define NUMBER_UNKNOWN	0

#define SIDE_UNKNOWN	0
#define SIDE_LEFT		1
#define SIDE_RIGHT		2

// constants for CGoal class

#define GOAL_UNKNOWN	0
#define GOAL_LEFT		1
#define GOAL_RIGHT		2

// constants for CFlag class

#define FLAG_UNKNOWN	0

#define FLAG_GOAL		1
#define FLAG_PENALTY	2
#define FLAG_FIELD		3

#define FLAG_LEFT		4
#define FLAG_RIGHT		5
#define FLAG_TOP		6
#define FLAG_BOTTOM		7
#define FLAG_CENTER		8

// constants for CLine class

#define LINE_UNKNOWN	0

#define LINE_LEFT		1
#define LINE_RIGHT		2
#define LINE_TOP		3
#define LINE_BOTTOM		4

// constants ???

//#define GOALSIZE			10
//#define VIEWPLANE_DIST	30.0
//#define EYE_HEIGHT		2.0
//#define SCALE_DIST		50.0

//#define MAX_IMAGINE_TIME	30		// time, a player can trace an invisible object in mind

//#define MAX_SPEED_DELTATIME  5      // time, a player can calculate speed of an invisible object

///////////////////////////////////////////////////////////////////////////////
// Class: CVisibleObject
// Date: 13.04.01
// Version Date: 13.04.01
// Version: 1.00
// Author: Marek Lehuta
//
// Base Class: CObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about visible objects
///////////////////////////////////////////////////////////////////////////////

class CVisibleObject : public CObject
{
// Construction
public:
	CVisibleObject() : m_absPos(0,0)  { m_bValid = false; };
	CVisibleObject(double dAbsX, double dAbsY);
	CVisibleObject(const CPosition& pos) : m_absPos(pos) { m_bValid = true; };
	CVisibleObject(const CVector& vect) : m_relPos(vect) { m_bValid = true; };
	CVisibleObject(const CVisibleObject& obj);

// Attributes
protected:
	bool m_bValid;				// true, if this object has valid information yet
	CPosition m_absPos;			// absolute position of the object
	CVector m_relPos;			// relative position of the object (from agent)

// Operations
public:
	void Invalidate() { m_bValid = false; };
	bool IsValid() const { return m_bValid; };

	void SetAbsX(double dX) { m_absPos.SetX(dX); };
	void SetAbsY(double dY) { m_absPos.SetY(dY); };
	void SetAbsPos(double dAbsX, double dAbsY);
	void SetAbsPos(const CPosition& pos) { m_absPos = pos; m_bValid = true; };
	void SetDistance(double dDist) { m_relPos.SetLength(dDist); };
	void SetDirection(double dDir) { m_relPos.SetAngle(dDir); };
	void SetRelX(double dX) { m_relPos.SetX(dX); };
	void SetRelY(double dY) { m_relPos.SetY(dY); };
	void SetRelVector(double dDist, double dDir);
	void SetRelPos(double dRelX, double dRelY);
	void SetRelPos(const CVector& vect) { m_relPos = vect; m_bValid = true; };
	double GetAbsX() const { return m_absPos.GetX(); };
	double GetAbsY() const { return m_absPos.GetY(); };
	CPosition GetAbsPos() const { return m_absPos; };
	double GetDistance() const { return m_relPos.GetLength(); };
	double GetDirection() const { return m_relPos.GetAngle(); };
	double GetRelX() const { return m_relPos.GetX(); };
	double GetRelY() const { return m_relPos.GetY(); };
	CVector GetRelPos() const { return m_relPos; };

// Overrides
public:
	virtual const CVisibleObject& operator=(const CVisibleObject& obj);
	virtual void ParseInfo(CParseString& str);

// Implementation
public:
	virtual ~CVisibleObject() {};

};

///////////////////////////////////////////////////////////////////////////////
// Class: CMovableObject
// Date: 13.04.01
// Version Date: 29.04.01
// Version: 1.13
// Author: Marek Lehuta
//
// Base Class: CVisibleObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about movable objects
///////////////////////////////////////////////////////////////////////////////

class CMovableObject : public CVisibleObject
{
// Construction
public:
	CMovableObject() { Init(); };
	CMovableObject(double dAbsX, double dAbsY);
	CMovableObject(const CPosition& aPos) : CVisibleObject(aPos) { Init(); };
	CMovableObject(const CVector& rPos) : CVisibleObject(rPos) { Init(); };
	CMovableObject(const CPosition& aPos, double dDistChng, double dDirChng);
	CMovableObject(const CVector& rPos, double dDistChng, double dDirChng);
	CMovableObject(const CPosition& aPos, const CVector& speed);
	CMovableObject(const CMovableObject& obj);
private:
	void Init();

// Attributes
protected:
	bool m_bLock;				// true, if this object was just parsed
	int m_iSeeTime;				// time of last perception
	int m_iSimTime;				// simulation time

	bool m_bChngValid;			// true, if see message contained change parameters
	double m_dDistChng;			// change of distance (perceived from server)
	double m_dDirChng;			// change of direction (perceived from server)

	CPosition m_oldPos;			// old absolute position of the object
	CVector m_speed;

// Operations
public:
	void Unlock() { m_bLock = false; };
	bool IsLocked() const { return m_bLock; };

	void SetTime(int iTime);
	int GetSeeTime() const { return m_iSeeTime; };
	int GetSimTime() const { return m_iSimTime; };
	int GetAge() const { return (m_iSimTime - m_iSeeTime); };

	bool IsChngValid() const { return m_bChngValid; };

	void SetDistChng(double dDistChng) { m_dDistChng = dDistChng; };
	void SetDirChng(double dDirChng) { m_dDirChng = dDirChng; };
	double GetDistChng() const { return m_dDistChng; };
	double GetDirChng() const { return m_dDirChng; };

	void SetAbsPos(double dAbsX, double dAbsY);
	void SetAbsPos(const CPosition& pos);
	void SetSpeedDist(double dDist) { m_speed.SetLength(dDist); };
	void SetSpeedDir(double dDir) { m_speed.SetAngle(dDir); };
	void SetSpeedX(double dSpX) { m_speed.SetX(dSpX); };
	void SetSpeedY(double dSpY) { m_speed.SetY(dSpY); };
	void SetSpeedVector(double dDist, double dDir);
	void SetSpeed(double dSpX, double dSpY);
	void SetSpeed(const CVector& speed);
	double GetOldPosX() const { return m_oldPos.GetX(); };
	double GetOldPosY() const { return m_oldPos.GetY(); };
	CPosition GetOldPos() const { return m_oldPos; };
	double GetSpeedDist() const { return m_speed.GetLength(); };
	double GetSpeedDir() const { return m_speed.GetAngle(); };
	double GetSpeedX() const { return m_speed.GetX(); };
	double GetSpeedY() const { return m_speed.GetY(); };
	CVector GetSpeed() const { return m_speed; };

	double GetDistanceTo(const CVisibleObject* pObj) const;
	double GetDistanceTo(const CPosition& pos) const;
	CPosition GetFuturePos(int iTime) const;

	void UpdatePos(const CPosition& refPos);
	void UpdateState(const CMovableObject& obj);

	void CalcAbsPos(const CPosition& refPos);
	void CalcSpeed(const CPosition& refPos);

// Overrides
public:
	virtual const CMovableObject& operator=(const CMovableObject& obj);
	virtual void ParseInfo(CParseString& str);
	virtual int GetMaxAge() const = 0;
	virtual double GetDecay() const = 0;
	virtual double GetSpeedMax() const = 0;

// Implementation
public:
	virtual ~CMovableObject() {};

};

//	CPosition m_absSeePos;		// absolute position where the object was seen
//	void SetLastSeePosition() { m_tLastSeePosition = GetPosition(); };
//	CVector& GetLastSeePosition() { return m_tLastSeePosition; };

//	TVector m_tRelSpeed;
//	TVector GetRelSpeed(TVector& pos, int dir);

//	BOOL	CheckPreventOffside(TPlayer* other);

//	double m_dPassDegree;			// quality of freeness - added by MP 24.04.99
//	TDirection m_iFreeAreaBordersLeft; //added by MP 24.04.99
//	TDirection m_iFreeAreaBordersRight; //added by MP 24.04.99
//	void	UpdateFreePassArea(TDirection pDir, int iSecureBoundary); //added by MP 06.05.99
//	int	GetFreePassArea(); // added by MP 24.04.99
//	int	GetFreePassDirection(); // added by MP 24.04.99
//	void	SetFreePassDegree(double dQuality){	m_dPassDegree = dQuality; }; // added by MP 24.04.99
//	double GetFreePassDegree() const { return m_dPassDegree; }; 
//	void	InitFreePassArea(int iSecureBorder); //added by MP 24.04.99

//	int			m_iHearTimePlayer;
//	int			m_iStamina;				//added by MP 27.05.99
//	BOOL	IsOnLeftSideOfField();
//	BOOL	IsInOwnHalfOfField();
//	BOOL	IsTired(void) const { if (m_iStamina < LOW_STAMINA) return FALSE; else return TRUE; };
//	void			SetHearTimePlayer(int time) { m_iHearTimePlayer = time; };
//	int			GetHearTimePlayer() const { return m_iHearTimePlayer; };
//	void			SetStamina(int val) { m_iStamina = val; }; //added by MP 27.05.99
//	int			GetStamina() { return m_iStamina; }; //added by MP 27.05.99

///////////////////////////////////////////////////////////////////////////////
// Class: CPlayer
// Date: 14.04.01
// Version Date: 29.04.01
// Version: 1.08
// Author: Marek Lehuta
//
// Base Class: CMovableObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about soccer players
///////////////////////////////////////////////////////////////////////////////

class CPlayer : public CMovableObject
{
// Construction
public:
	CPlayer();
	CPlayer(const CPlayer& player);

// Attributes
protected:
	int m_iSide;				// side the player is on (SIDE_LEFT, ...)
	int m_iNumber;				// number of the player (1 - 11)

	bool m_bDirValid;			// true, if see message contained direction parameters
	double m_dBodyDir;			// direction the player is running
	double m_dHeadDir;			// direction the player is looking (relative to body)

// Operations
public:
	void SetSide(int iSide) { m_iSide = iSide; };
	void SetNumber(int iNumber) { m_iNumber = iNumber; };
	int GetSide() const { return m_iSide; };
	int GetNumber() const { return m_iNumber; };

	bool IsDirValid() const { return m_bDirValid; };

	void SetBodyDir(double dBodyDir);
	void SetHeadDir(double dHeadDir);
	double GetBodyDir() const { return m_dBodyDir; };
	double GetHeadDir() const { return m_dHeadDir; };

	double GetAngleTo(const CVisibleObject* pObj) const;
	double GetAngleTo(const CPosition& pos) const;
	double GetHeadAngleTo(const CVisibleObject* pObj) const;
	double GetHeadAngleTo(const CPosition& pos) const;
//	CPosition GetNextFuturePosition() const;
	double GetNextFutureDistance() const;
//	CPosition GetFuturePositionActive(int iTime, double dPower, double dDir, double dEffort) const;
	double GetFutureDistanceActive(int iTime, double dPower, double dDir, double dEffort) const;
  double GetFutureDistanceActive(int iTime, double dPower, bool bIsTurn) const;

	void UpdateState(const CPlayer& player);

// Overrides
public:
	virtual const CPlayer& operator=(const CPlayer& player);
	virtual void ParseInfo(CParseString& str);
	virtual int GetMaxAge() const { return PLAYER_MAX_AGE; };
	virtual double GetDecay() const { return PLAYER_DECAY; };
	virtual double GetSpeedMax() const { return PLAYER_SPEED_MAX; };

protected:

// Implementation
public:
	virtual ~CPlayer() {};

};

///////////////////////////////////////////////////////////////////////////////
// Class: CBall
// Date: 14.04.01
// Version Date: 25.04.01
// Version: 1.03
// Author: Marek Lehuta
//
// Base Class: CMovableObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about soccer ball
///////////////////////////////////////////////////////////////////////////////

class CBall : public CMovableObject
{
// Construction
public:
	CBall() {};
	CBall(const CBall& ball) : CMovableObject(ball) {};

// Attributes
protected:
	int m_iOur;

// Operations
public:
	void SetOur(int iOur) { m_iOur = iOur; };
	int GetOur() const { return m_iOur; };

// Overrides
public:
	virtual const CBall& operator=(const CBall& ball);
	virtual void ParseInfo(CParseString& str);
	virtual int GetMaxAge() const { return BALL_MAX_AGE; };
	virtual double GetDecay() const { return BALL_DECAY; };
	virtual double GetSpeedMax() const { return BALL_SPEED_MAX; };

// Implementation
public:
	virtual ~CBall() {};

};

///////////////////////////////////////////////////////////////////////////////
// Class: CGoal
// Date: 14.04.01
// Version Date: 14.04.01
// Version: 1.00
// Author: Marek Lehuta
//
// Base Class: CVisibleObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about soccer goals
///////////////////////////////////////////////////////////////////////////////

class CGoal : public CVisibleObject
{
// Construction
public:
	CGoal() { m_iID = GOAL_UNKNOWN; };
	CGoal(int iID) { m_iID = iID; CalcAbsPos(); };
	CGoal(const CGoal& goal) : CVisibleObject(goal) { m_iID = goal.GetID(); };

// Attributes
protected:
	int m_iID;					// identification of a goal

// Operations
public:
	void SetID(int iID) { m_iID = iID; CalcAbsPos(); };
	int GetID() const { return m_iID; };

private:
	void CalcAbsPos();

// Overrides
public:
	virtual const CGoal& operator=(const CGoal& goal);
	virtual void ParseInfo(CParseString& str);

// Implementation
public:
	virtual ~CGoal() {};

};

///////////////////////////////////////////////////////////////////////////////
// Class: CFlag
// Date: 14.04.01
// Version Date: 14.04.01
// Version: 1.00
// Author: Marek Lehuta
//
// Base Class: CVisibleObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about soccer field flags
///////////////////////////////////////////////////////////////////////////////

class CFlag : public CVisibleObject
{
// Construction
public:
	CFlag() { m_iAreaID = m_iPriID = m_iSecID = m_iTerID = FLAG_UNKNOWN; };
	CFlag(int iAreaID, int iPriID, int iSecID, int iTerID);
	CFlag(const CFlag& flag);

// Attributes
protected:
	int m_iAreaID;				// area identification of a flag (penalty area, ...)
	int m_iPriID;				// primary identification of a flag (left, ...)
	int m_iSecID;				// secondary identification of a flag (left, ...)
	int m_iTerID;				// tertiary identification of a flag (0 - 50)

// Operations
public:
	void SetID(int iAreaID, int iPriID, int iSecID, int iTerID);
	int GetAreaID() const { return m_iAreaID; };
	int GetPriID() const { return m_iPriID; };
	int GetSecID() const { return m_iSecID; };
	int GetTerID() const { return m_iTerID; };

private:
	void CalcAbsPos();

// Overrides
public:
	virtual const CFlag& operator=(const CFlag& flag);
	virtual void ParseInfo(CParseString& str);

// Implementation
public:
	virtual ~CFlag() {};

};

///////////////////////////////////////////////////////////////////////////////
// Class: CLine
// Date: 14.04.01
// Version Date: 14.04.01
// Version: 1.00
// Author: Marek Lehuta
//
// Base Class: CVisibleObject
///////////////////////////////////////////////////////////////////////////////
// Description: contains information about soccer field lines
///////////////////////////////////////////////////////////////////////////////

class CLine : public CVisibleObject
{
// Construction
public:
	CLine() { m_iID = LINE_UNKNOWN; };
	CLine(int iID) { m_iID = iID; CalcAbsPos(); };
	CLine(const CLine& line);

// Attributes
protected:
	int m_iID;					// identification of a line
	CPosition m_endPos;			// end position of the line (used for calculation of vector)

// Operations
public:
	void SetID(int iID) { m_iID = iID; CalcAbsPos(); };
	int GetID() const { return m_iID; };

	void SetEndPosX(double dX) { m_endPos.SetX(dX); };
	void SetEndPosY(double dY) { m_endPos.SetY(dY); };
	void SetEndPos(double dEndX, double dEndY);
	void SetEndPos(const CPosition& pos) { m_endPos = pos; m_bValid = true; };
	double GetEndPosX() const { return m_endPos.GetX(); };
	double GetEndPosY() const { return m_endPos.GetY(); };
	CPosition GetEndPos() const { return m_endPos; };
	CVector GetLineVector() const { return m_endPos - m_absPos; };

private:
	void CalcAbsPos();

// Overrides
public:
	virtual const CLine& operator=(const CLine& line);
	virtual void ParseInfo(CParseString& str);

// Implementation
public:
	virtual ~CLine() {};

};

#endif	// __AFX_VISIBLEOBJECT_H_
